
From: Dominik Brodowski <linux@dominikbrodowski.net>

The linked list of devices managed by each PCMCIA driver is, in very most
cases, unused.  Therefore, remove it from many drivers.

Signed-off-by: Dominik Brodowski <linux@dominikbrodowski.net>
Signed-off-by: Andrew Morton <akpm@osdl.org>
---

 drivers/bluetooth/bluecard_cs.c         |   18 ----------
 drivers/bluetooth/bt3c_cs.c             |   18 ----------
 drivers/bluetooth/btuart_cs.c           |   18 ----------
 drivers/bluetooth/dtl1_cs.c             |   18 ----------
 drivers/char/pcmcia/synclink_cs.c       |   14 --------
 drivers/ide/legacy/ide-cs.c             |   19 ++--------
 drivers/isdn/hardware/avm/avm_cs.c      |   26 +-------------
 drivers/isdn/hisax/avma1_cs.c           |   27 ++-------------
 drivers/isdn/hisax/elsa_cs.c            |   28 +---------------
 drivers/isdn/hisax/sedlbauer_cs.c       |   28 +---------------
 drivers/isdn/hisax/teles_cs.c           |   28 +---------------
 drivers/mtd/maps/pcmciamtd.c            |    5 --
 drivers/net/pcmcia/3c574_cs.c           |   15 --------
 drivers/net/pcmcia/3c589_cs.c           |   17 +--------
 drivers/net/pcmcia/axnet_cs.c           |   14 --------
 drivers/net/pcmcia/com20020_cs.c        |   22 ++----------
 drivers/net/pcmcia/fmvj18x_cs.c         |   16 +--------
 drivers/net/pcmcia/ibmtr_cs.c           |   21 +-----------
 drivers/net/pcmcia/nmclan_cs.c          |   14 --------
 drivers/net/pcmcia/pcnet_cs.c           |   14 --------
 drivers/net/pcmcia/smc91c92_cs.c        |   15 --------
 drivers/net/pcmcia/xirc2ps_cs.c         |   24 -------------
 drivers/net/wireless/airo_cs.c          |   28 +++-------------
 drivers/net/wireless/atmel_cs.c         |   25 +-------------
 drivers/net/wireless/hostap/hostap_cs.c |   15 --------
 drivers/net/wireless/netwave_cs.c       |   56 ++++++--------------------------
 drivers/net/wireless/orinoco_cs.c       |   21 ------------
 drivers/net/wireless/wavelan_cs.c       |   24 -------------
 drivers/net/wireless/wavelan_cs.p.h     |    1 
 drivers/parport/parport_cs.c            |   14 --------
 drivers/scsi/pcmcia/fdomain_stub.c      |   30 ++++-------------
 drivers/scsi/pcmcia/nsp_cs.c            |   19 ----------
 drivers/scsi/pcmcia/qlogic_stub.c       |   16 ---------
 drivers/scsi/pcmcia/sym53c500_cs.c      |   14 --------
 drivers/serial/serial_cs.c              |   17 +--------
 drivers/telephony/ixj_pcmcia.c          |   14 +-------
 drivers/usb/host/sl811_cs.c             |   16 ---------
 sound/pcmcia/pdaudiocf/pdaudiocf.c      |   16 +--------
 sound/pcmcia/vx/vxpocket.c              |   13 -------
 39 files changed, 87 insertions(+), 671 deletions(-)

diff -puN drivers/bluetooth/bluecard_cs.c~pcmcia-remove-unused-dev_list-in-drivers drivers/bluetooth/bluecard_cs.c
--- devel/drivers/bluetooth/bluecard_cs.c~pcmcia-remove-unused-dev_list-in-drivers	2005-09-12 03:34:43.000000000 -0700
+++ devel-akpm/drivers/bluetooth/bluecard_cs.c	2005-09-12 03:34:43.000000000 -0700
@@ -94,8 +94,6 @@ static dev_info_t dev_info = "bluecard_c
 static dev_link_t *bluecard_attach(void);
 static void bluecard_detach(struct pcmcia_device *p_dev);
 
-static dev_link_t *dev_list = NULL;
-
 
 /* Default baud rate: 57600, 115200, 230400 or 460800 */
 #define DEFAULT_BAUD_RATE  230400
@@ -891,8 +889,7 @@ static dev_link_t *bluecard_attach(void)
 	link->conf.IntType = INT_MEMORY_AND_IO;
 
 	/* Register with Card Services */
-	link->next = dev_list;
-	dev_list = link;
+	link->next = NULL;
 	client_reg.dev_info = &dev_info;
 	client_reg.Version = 0x0210;
 	client_reg.event_callback_args.client_data = link;
@@ -912,22 +909,10 @@ static void bluecard_detach(struct pcmci
 {
 	dev_link_t *link = dev_to_instance(p_dev);
 	bluecard_info_t *info = link->priv;
-	dev_link_t **linkp;
-
-	/* Locate device structure */
-	for (linkp = &dev_list; *linkp; linkp = &(*linkp)->next)
-		if (*linkp == link)
-			break;
-
-	if (*linkp == NULL)
-		return;
 
 	if (link->state & DEV_CONFIG)
 		bluecard_release(link);
 
-	/* Unlink device structure, free bits */
-	*linkp = link->next;
-
 	kfree(info);
 }
 
@@ -1106,7 +1091,6 @@ static int __init init_bluecard_cs(void)
 static void __exit exit_bluecard_cs(void)
 {
 	pcmcia_unregister_driver(&bluecard_driver);
-	BUG_ON(dev_list != NULL);
 }
 
 module_init(init_bluecard_cs);
diff -puN drivers/bluetooth/bt3c_cs.c~pcmcia-remove-unused-dev_list-in-drivers drivers/bluetooth/bt3c_cs.c
--- devel/drivers/bluetooth/bt3c_cs.c~pcmcia-remove-unused-dev_list-in-drivers	2005-09-12 03:34:43.000000000 -0700
+++ devel-akpm/drivers/bluetooth/bt3c_cs.c	2005-09-12 03:34:43.000000000 -0700
@@ -97,8 +97,6 @@ static dev_info_t dev_info = "bt3c_cs";
 static dev_link_t *bt3c_attach(void);
 static void bt3c_detach(struct pcmcia_device *p_dev);
 
-static dev_link_t *dev_list = NULL;
-
 
 /* Transmit states  */
 #define XMIT_SENDING  1
@@ -692,8 +690,7 @@ static dev_link_t *bt3c_attach(void)
 	link->conf.IntType = INT_MEMORY_AND_IO;
 
 	/* Register with Card Services */
-	link->next = dev_list;
-	dev_list = link;
+	link->next = NULL;
 	client_reg.dev_info = &dev_info;
 	client_reg.Version = 0x0210;
 	client_reg.event_callback_args.client_data = link;
@@ -713,22 +710,10 @@ static void bt3c_detach(struct pcmcia_de
 {
 	dev_link_t *link = dev_to_instance(p_dev);
 	bt3c_info_t *info = link->priv;
-	dev_link_t **linkp;
-
-	/* Locate device structure */
-	for (linkp = &dev_list; *linkp; linkp = &(*linkp)->next)
-		if (*linkp == link)
-			break;
-
-	if (*linkp == NULL)
-		return;
 
 	if (link->state & DEV_CONFIG)
 		bt3c_release(link);
 
-	/* Unlink device structure, free bits */
-	*linkp = link->next;
-
 	kfree(info);
 }
 
@@ -950,7 +935,6 @@ static int __init init_bt3c_cs(void)
 static void __exit exit_bt3c_cs(void)
 {
 	pcmcia_unregister_driver(&bt3c_driver);
-	BUG_ON(dev_list != NULL);
 }
 
 module_init(init_bt3c_cs);
diff -puN drivers/bluetooth/btuart_cs.c~pcmcia-remove-unused-dev_list-in-drivers drivers/bluetooth/btuart_cs.c
--- devel/drivers/bluetooth/btuart_cs.c~pcmcia-remove-unused-dev_list-in-drivers	2005-09-12 03:34:43.000000000 -0700
+++ devel-akpm/drivers/bluetooth/btuart_cs.c	2005-09-12 03:34:43.000000000 -0700
@@ -93,8 +93,6 @@ static dev_info_t dev_info = "btuart_cs"
 static dev_link_t *btuart_attach(void);
 static void btuart_detach(struct pcmcia_device *p_dev);
 
-static dev_link_t *dev_list = NULL;
-
 
 /* Maximum baud rate */
 #define SPEED_MAX  115200
@@ -611,8 +609,7 @@ static dev_link_t *btuart_attach(void)
 	link->conf.IntType = INT_MEMORY_AND_IO;
 
 	/* Register with Card Services */
-	link->next = dev_list;
-	dev_list = link;
+	link->next = NULL;
 	client_reg.dev_info = &dev_info;
 	client_reg.Version = 0x0210;
 	client_reg.event_callback_args.client_data = link;
@@ -632,22 +629,10 @@ static void btuart_detach(struct pcmcia_
 {
 	dev_link_t *link = dev_to_instance(p_dev);
 	btuart_info_t *info = link->priv;
-	dev_link_t **linkp;
-
-	/* Locate device structure */
-	for (linkp = &dev_list; *linkp; linkp = &(*linkp)->next)
-		if (*linkp == link)
-			break;
-
-	if (*linkp == NULL)
-		return;
 
 	if (link->state & DEV_CONFIG)
 		btuart_release(link);
 
-	/* Unlink device structure, free bits */
-	*linkp = link->next;
-
 	kfree(info);
 }
 
@@ -871,7 +856,6 @@ static int __init init_btuart_cs(void)
 static void __exit exit_btuart_cs(void)
 {
 	pcmcia_unregister_driver(&btuart_driver);
-	BUG_ON(dev_list != NULL);
 }
 
 module_init(init_btuart_cs);
diff -puN drivers/bluetooth/dtl1_cs.c~pcmcia-remove-unused-dev_list-in-drivers drivers/bluetooth/dtl1_cs.c
--- devel/drivers/bluetooth/dtl1_cs.c~pcmcia-remove-unused-dev_list-in-drivers	2005-09-12 03:34:43.000000000 -0700
+++ devel-akpm/drivers/bluetooth/dtl1_cs.c	2005-09-12 03:34:43.000000000 -0700
@@ -96,8 +96,6 @@ static dev_info_t dev_info = "dtl1_cs";
 static dev_link_t *dtl1_attach(void);
 static void dtl1_detach(struct pcmcia_device *p_dev);
 
-static dev_link_t *dev_list = NULL;
-
 
 /* Transmit states  */
 #define XMIT_SENDING  1
@@ -590,8 +588,7 @@ static dev_link_t *dtl1_attach(void)
 	link->conf.IntType = INT_MEMORY_AND_IO;
 
 	/* Register with Card Services */
-	link->next = dev_list;
-	dev_list = link;
+	link->next = NULL;
 	client_reg.dev_info = &dev_info;
 	client_reg.Version = 0x0210;
 	client_reg.event_callback_args.client_data = link;
@@ -611,22 +608,10 @@ static void dtl1_detach(struct pcmcia_de
 {
 	dev_link_t *link = dev_to_instance(p_dev);
 	dtl1_info_t *info = link->priv;
-	dev_link_t **linkp;
-
-	/* Locate device structure */
-	for (linkp = &dev_list; *linkp; linkp = &(*linkp)->next)
-		if (*linkp == link)
-			break;
-
-	if (*linkp == NULL)
-		return;
 
 	if (link->state & DEV_CONFIG)
 		dtl1_release(link);
 
-	/* Unlink device structure, free bits */
-	*linkp = link->next;
-
 	kfree(info);
 }
 
@@ -823,7 +808,6 @@ static int __init init_dtl1_cs(void)
 static void __exit exit_dtl1_cs(void)
 {
 	pcmcia_unregister_driver(&dtl1_driver);
-	BUG_ON(dev_list != NULL);
 }
 
 module_init(init_dtl1_cs);
diff -puN drivers/char/pcmcia/synclink_cs.c~pcmcia-remove-unused-dev_list-in-drivers drivers/char/pcmcia/synclink_cs.c
--- devel/drivers/char/pcmcia/synclink_cs.c~pcmcia-remove-unused-dev_list-in-drivers	2005-09-12 03:34:43.000000000 -0700
+++ devel-akpm/drivers/char/pcmcia/synclink_cs.c	2005-09-12 03:34:43.000000000 -0700
@@ -492,7 +492,6 @@ static dev_link_t *mgslpc_attach(void);
 static void mgslpc_detach(struct pcmcia_device *p_dev);
 
 static dev_info_t dev_info = "synclink_cs";
-static dev_link_t *dev_list = NULL;
 
 /*
  * 1st function defined in .text section. Calling this function in
@@ -588,8 +587,7 @@ static dev_link_t *mgslpc_attach(void)
     link->conf.IntType = INT_MEMORY_AND_IO;
 
     /* Register with Card Services */
-    link->next = dev_list;
-    dev_list = link;
+    link->next = NULL;
 
     client_reg.dev_info = &dev_info;
     client_reg.Version = 0x0210;
@@ -741,24 +739,15 @@ static void mgslpc_release(u_long arg)
 static void mgslpc_detach(struct pcmcia_device *p_dev)
 {
     dev_link_t *link = dev_to_instance(p_dev);
-    dev_link_t **linkp;
 
     if (debug_level >= DEBUG_LEVEL_INFO)
 	    printk("mgslpc_detach(0x%p)\n", link);
 
-    /* find device */
-    for (linkp = &dev_list; *linkp; linkp = &(*linkp)->next)
-	    if (*linkp == link) break;
-    if (*linkp == NULL)
-	    return;
-
     if (link->state & DEV_CONFIG) {
 	    ((MGSLPC_INFO *)link->priv)->stop = 1;
 	    mgslpc_release((u_long)link);
     }
 
-    /* Unlink device structure, and free it */
-    *linkp = link->next;
     mgslpc_remove_device((MGSLPC_INFO *)link->priv);
 }
 
@@ -3132,7 +3121,6 @@ static void synclink_cs_cleanup(void)
 	}
 
 	pcmcia_unregister_driver(&mgslpc_driver);
-	BUG_ON(dev_list != NULL);
 }
 
 static int __init synclink_cs_init(void)
diff -puN drivers/ide/legacy/ide-cs.c~pcmcia-remove-unused-dev_list-in-drivers drivers/ide/legacy/ide-cs.c
--- devel/drivers/ide/legacy/ide-cs.c~pcmcia-remove-unused-dev_list-in-drivers	2005-09-12 03:34:43.000000000 -0700
+++ devel-akpm/drivers/ide/legacy/ide-cs.c	2005-09-12 03:34:43.000000000 -0700
@@ -96,7 +96,8 @@ static dev_info_t dev_info = "ide-cs";
 static dev_link_t *ide_attach(void);
 static void ide_detach(struct pcmcia_device *p_dev);
 
-static dev_link_t *dev_list = NULL;
+
+
 
 /*======================================================================
 
@@ -131,8 +132,7 @@ static dev_link_t *ide_attach(void)
     link->conf.IntType = INT_MEMORY_AND_IO;
     
     /* Register with Card Services */
-    link->next = dev_list;
-    dev_list = link;
+    link->next = NULL;
     client_reg.dev_info = &dev_info;
     client_reg.Version = 0x0210;
     client_reg.event_callback_args.client_data = link;
@@ -158,23 +158,13 @@ static dev_link_t *ide_attach(void)
 static void ide_detach(struct pcmcia_device *p_dev)
 {
     dev_link_t *link = dev_to_instance(p_dev);
-    dev_link_t **linkp;
 
     DEBUG(0, "ide_detach(0x%p)\n", link);
-    
-    /* Locate device structure */
-    for (linkp = &dev_list; *linkp; linkp = &(*linkp)->next)
-	if (*linkp == link) break;
-    if (*linkp == NULL)
-	return;
 
     if (link->state & DEV_CONFIG)
 	ide_release(link);
-    
-    /* Unlink, free device structure */
-    *linkp = link->next;
+
     kfree(link->priv);
-    
 } /* ide_detach */
 
 static int idecs_register(unsigned long io, unsigned long ctl, unsigned long irq)
@@ -508,7 +498,6 @@ static int __init init_ide_cs(void)
 static void __exit exit_ide_cs(void)
 {
 	pcmcia_unregister_driver(&ide_cs_driver);
-	BUG_ON(dev_list != NULL);
 }
 
 late_initcall(init_ide_cs);
diff -puN drivers/isdn/hardware/avm/avm_cs.c~pcmcia-remove-unused-dev_list-in-drivers drivers/isdn/hardware/avm/avm_cs.c
--- devel/drivers/isdn/hardware/avm/avm_cs.c~pcmcia-remove-unused-dev_list-in-drivers	2005-09-12 03:34:43.000000000 -0700
+++ devel-akpm/drivers/isdn/hardware/avm/avm_cs.c	2005-09-12 03:34:43.000000000 -0700
@@ -83,15 +83,7 @@ static dev_info_t dev_info = "avm_cs";
    device numbers are used to derive the corresponding array index.
 */
 
-static dev_link_t *dev_list = NULL;
-
 /*
-   A dev_link_t structure has fields for most things that are needed
-   to keep track of a socket, but there will usually be some device
-   specific information that also needs to be kept track of.  The
-   'priv' pointer in a dev_link_t structure can be used to point to
-   a device-specific private data structure, like this.
-
    A driver needs to provide a dev_node_t structure for each device
    on a card.  In some cases, there is only one device per card (for
    example, ethernet cards, modems).  In other cases, there may be
@@ -157,8 +149,7 @@ static dev_link_t *avmcs_attach(void)
     link->priv = local;
     
     /* Register with Card Services */
-    link->next = dev_list;
-    dev_list = link;
+    link->next = NULL;
     client_reg.dev_info = &dev_info;
     client_reg.Version = 0x0210;
     client_reg.event_callback_args.client_data = link;
@@ -188,23 +179,13 @@ static dev_link_t *avmcs_attach(void)
 static void avmcs_detach(struct pcmcia_device *p_dev)
 {
     dev_link_t *link = dev_to_instance(p_dev);
-    dev_link_t **linkp;
-
-    /* Locate device structure */
-    for (linkp = &dev_list; *linkp; linkp = &(*linkp)->next)
-	if (*linkp == link) break;
-    if (*linkp == NULL)
-	return;
 
     if (link->state & DEV_CONFIG)
         avmcs_release(link);
 
+    if (link->priv)
+	    kfree(link->priv);
 
-    /* Unlink device structure, free pieces */
-    *linkp = link->next;
-    if (link->priv) {
-	kfree(link->priv);
-    }
     kfree(link);
 } /* avmcs_detach */
 
@@ -497,7 +478,6 @@ static int __init avmcs_init(void)
 static void __exit avmcs_exit(void)
 {
 	pcmcia_unregister_driver(&avmcs_driver);
-	BUG_ON(dev_list != NULL);
 }
 
 module_init(avmcs_init);
diff -puN drivers/isdn/hisax/avma1_cs.c~pcmcia-remove-unused-dev_list-in-drivers drivers/isdn/hisax/avma1_cs.c
--- devel/drivers/isdn/hisax/avma1_cs.c~pcmcia-remove-unused-dev_list-in-drivers	2005-09-12 03:34:43.000000000 -0700
+++ devel-akpm/drivers/isdn/hisax/avma1_cs.c	2005-09-12 03:34:43.000000000 -0700
@@ -99,15 +99,7 @@ static dev_info_t dev_info = "avma1_cs";
    device numbers are used to derive the corresponding array index.
 */
 
-static dev_link_t *dev_list = NULL;
-
 /*
-   A dev_link_t structure has fields for most things that are needed
-   to keep track of a socket, but there will usually be some device
-   specific information that also needs to be kept track of.  The
-   'priv' pointer in a dev_link_t structure can be used to point to
-   a device-specific private data structure, like this.
-
    A driver needs to provide a dev_node_t structure for each device
    on a card.  In some cases, there is only one device per card (for
    example, ethernet cards, modems).  In other cases, there may be
@@ -179,8 +171,7 @@ static dev_link_t *avma1cs_attach(void)
     link->conf.Present = PRESENT_OPTION;
 
     /* Register with Card Services */
-    link->next = dev_list;
-    dev_list = link;
+    link->next = NULL;
     client_reg.dev_info = &dev_info;
     client_reg.Version = 0x0210;
     client_reg.event_callback_args.client_data = link;
@@ -206,24 +197,15 @@ static dev_link_t *avma1cs_attach(void)
 static void avma1cs_detach(struct pcmcia_device *p_dev)
 {
     dev_link_t *link = dev_to_instance(p_dev);
-    dev_link_t **linkp;
 
     DEBUG(0, "avma1cs_detach(0x%p)\n", link);
 
-    /* Locate device structure */
-    for (linkp = &dev_list; *linkp; linkp = &(*linkp)->next)
-	if (*linkp == link) break;
-    if (*linkp == NULL)
-	return;
-
     if (link->state & DEV_CONFIG)
 	    avma1cs_release(link);
 
-    /* Unlink device structure, free pieces */
-    *linkp = link->next;
-    if (link->priv) {
-	kfree(link->priv);
-    }
+    if (link->priv)
+	    kfree(link->priv);
+
     kfree(link);
 } /* avma1cs_detach */
 
@@ -510,7 +492,6 @@ static int __init init_avma1_cs(void)
 static void __exit exit_avma1_cs(void)
 {
 	pcmcia_unregister_driver(&avma1cs_driver);
-	BUG_ON(dev_list != NULL);
 }
 
 module_init(init_avma1_cs);
diff -puN drivers/isdn/hisax/elsa_cs.c~pcmcia-remove-unused-dev_list-in-drivers drivers/isdn/hisax/elsa_cs.c
--- devel/drivers/isdn/hisax/elsa_cs.c~pcmcia-remove-unused-dev_list-in-drivers	2005-09-12 03:34:43.000000000 -0700
+++ devel-akpm/drivers/isdn/hisax/elsa_cs.c	2005-09-12 03:34:43.000000000 -0700
@@ -126,18 +126,7 @@ static dev_info_t dev_info = "elsa_cs";
    device numbers are used to derive the corresponding array index.
 */
 
-static dev_link_t *dev_list = NULL;
-
 /*
-   A dev_link_t structure has fields for most things that are needed
-   to keep track of a socket, but there will usually be some device
-   specific information that also needs to be kept track of.  The
-   'priv' pointer in a dev_link_t structure can be used to point to
-   a device-specific private data structure, like this.
-
-   To simplify the data structure handling, we actually include the
-   dev_link_t structure in the device's private data structure.
-
    A driver needs to provide a dev_node_t structure for each device
    on a card.  In some cases, there is only one device per card (for
    example, ethernet cards, modems).  In other cases, there may be
@@ -208,8 +197,7 @@ static dev_link_t *elsa_cs_attach(void)
     link->conf.IntType = INT_MEMORY_AND_IO;
 
     /* Register with Card Services */
-    link->next = dev_list;
-    dev_list = link;
+    link->next = NULL;
     client_reg.dev_info = &dev_info;
     client_reg.Version = 0x0210;
     client_reg.event_callback_args.client_data = link;
@@ -235,24 +223,15 @@ static dev_link_t *elsa_cs_attach(void)
 static void elsa_cs_detach(struct pcmcia_device *p_dev)
 {
     dev_link_t *link = dev_to_instance(p_dev);
-    dev_link_t **linkp;
     local_info_t *info = link->priv;
 
     DEBUG(0, "elsa_cs_detach(0x%p)\n", link);
 
-    /* Locate device structure */
-    for (linkp = &dev_list; *linkp; linkp = &(*linkp)->next)
-        if (*linkp == link) break;
-    if (*linkp == NULL)
-        return;
-
     if (link->state & DEV_CONFIG) {
-        ((local_info_t*)link->priv)->busy = 1;
-        elsa_cs_release(link);
+	    info->busy = 1;
+	    elsa_cs_release(link);
     }
 
-    /* Unlink device structure and free it */
-    *linkp = link->next;
     kfree(info);
 
 } /* elsa_cs_detach */
@@ -526,7 +505,6 @@ static int __init init_elsa_cs(void)
 static void __exit exit_elsa_cs(void)
 {
 	pcmcia_unregister_driver(&elsa_cs_driver);
-	BUG_ON(dev_list != NULL);
 }
 
 module_init(init_elsa_cs);
diff -puN drivers/isdn/hisax/sedlbauer_cs.c~pcmcia-remove-unused-dev_list-in-drivers drivers/isdn/hisax/sedlbauer_cs.c
--- devel/drivers/isdn/hisax/sedlbauer_cs.c~pcmcia-remove-unused-dev_list-in-drivers	2005-09-12 03:34:43.000000000 -0700
+++ devel-akpm/drivers/isdn/hisax/sedlbauer_cs.c	2005-09-12 03:34:43.000000000 -0700
@@ -134,18 +134,7 @@ static dev_info_t dev_info = "sedlbauer_
    device numbers are used to derive the corresponding array index.
 */
 
-static dev_link_t *dev_list = NULL;
-
 /*
-   A dev_link_t structure has fields for most things that are needed
-   to keep track of a socket, but there will usually be some device
-   specific information that also needs to be kept track of.  The
-   'priv' pointer in a dev_link_t structure can be used to point to
-   a device-specific private data structure, like this.
-
-   To simplify the data structure handling, we actually include the
-   dev_link_t structure in the device's private data structure.
-
    A driver needs to provide a dev_node_t structure for each device
    on a card.  In some cases, there is only one device per card (for
    example, ethernet cards, modems).  In other cases, there may be
@@ -222,8 +211,7 @@ static dev_link_t *sedlbauer_attach(void
     link->conf.IntType = INT_MEMORY_AND_IO;
 
     /* Register with Card Services */
-    link->next = dev_list;
-    dev_list = link;
+    link->next = NULL;
     client_reg.dev_info = &dev_info;
     client_reg.Version = 0x0210;
     client_reg.event_callback_args.client_data = link;
@@ -249,23 +237,14 @@ static dev_link_t *sedlbauer_attach(void
 static void sedlbauer_detach(struct pcmcia_device *p_dev)
 {
     dev_link_t *link = dev_to_instance(p_dev);
-    dev_link_t **linkp;
 
     DEBUG(0, "sedlbauer_detach(0x%p)\n", link);
-    
-    /* Locate device structure */
-    for (linkp = &dev_list; *linkp; linkp = &(*linkp)->next)
-	if (*linkp == link) break;
-    if (*linkp == NULL)
-	return;
 
     if (link->state & DEV_CONFIG) {
-	((local_info_t *)link->priv)->stop = 1;
-	sedlbauer_release(link);
+	    ((local_info_t *)link->priv)->stop = 1;
+	    sedlbauer_release(link);
     }
 
-    /* Unlink device structure, and free it */
-    *linkp = link->next;
     /* This points to the parent local_info_t struct */
     kfree(link->priv);
 } /* sedlbauer_detach */
@@ -623,7 +602,6 @@ static int __init init_sedlbauer_cs(void
 static void __exit exit_sedlbauer_cs(void)
 {
 	pcmcia_unregister_driver(&sedlbauer_driver);
-	BUG_ON(dev_list != NULL);
 }
 
 module_init(init_sedlbauer_cs);
diff -puN drivers/isdn/hisax/teles_cs.c~pcmcia-remove-unused-dev_list-in-drivers drivers/isdn/hisax/teles_cs.c
--- devel/drivers/isdn/hisax/teles_cs.c~pcmcia-remove-unused-dev_list-in-drivers	2005-09-12 03:34:43.000000000 -0700
+++ devel-akpm/drivers/isdn/hisax/teles_cs.c	2005-09-12 03:34:43.000000000 -0700
@@ -107,18 +107,7 @@ static dev_info_t dev_info = "teles_cs";
    device numbers are used to derive the corresponding array index.
 */
 
-static dev_link_t *dev_list = NULL;
-
 /*
-   A dev_link_t structure has fields for most things that are needed
-   to keep track of a socket, but there will usually be some device
-   specific information that also needs to be kept track of.  The
-   'priv' pointer in a dev_link_t structure can be used to point to
-   a device-specific private data structure, like this.
-
-   To simplify the data structure handling, we actually include the
-   dev_link_t structure in the device's private data structure.
-
    A driver needs to provide a dev_node_t structure for each device
    on a card.  In some cases, there is only one device per card (for
    example, ethernet cards, modems).  In other cases, there may be
@@ -189,8 +178,7 @@ static dev_link_t *teles_attach(void)
     link->conf.IntType = INT_MEMORY_AND_IO;
 
     /* Register with Card Services */
-    link->next = dev_list;
-    dev_list = link;
+    link->next = NULL;
     client_reg.dev_info = &dev_info;
     client_reg.Version = 0x0210;
     client_reg.event_callback_args.client_data = link;
@@ -216,24 +204,15 @@ static dev_link_t *teles_attach(void)
 static void teles_detach(struct pcmcia_device *p_dev)
 {
     dev_link_t *link = dev_to_instance(p_dev);
-    dev_link_t **linkp;
     local_info_t *info = link->priv;
 
     DEBUG(0, "teles_detach(0x%p)\n", link);
 
-    /* Locate device structure */
-    for (linkp = &dev_list; *linkp; linkp = &(*linkp)->next)
-        if (*linkp == link) break;
-    if (*linkp == NULL)
-        return;
-
     if (link->state & DEV_CONFIG) {
-        info->busy = 1;
-        teles_cs_release(link);
+	    info->busy = 1;
+	    teles_cs_release(link);
     }
 
-    /* Unlink device structure and free it */
-    *linkp = link->next;
     kfree(info);
 
 } /* teles_detach */
@@ -506,7 +485,6 @@ static int __init init_teles_cs(void)
 static void __exit exit_teles_cs(void)
 {
 	pcmcia_unregister_driver(&teles_cs_driver);
-	BUG_ON(dev_list != NULL);
 }
 
 module_init(init_teles_cs);
diff -puN drivers/mtd/maps/pcmciamtd.c~pcmcia-remove-unused-dev_list-in-drivers drivers/mtd/maps/pcmciamtd.c
--- devel/drivers/mtd/maps/pcmciamtd.c~pcmcia-remove-unused-dev_list-in-drivers	2005-09-12 03:34:43.000000000 -0700
+++ devel-akpm/drivers/mtd/maps/pcmciamtd.c	2005-09-12 03:34:43.000000000 -0700
@@ -67,7 +67,6 @@ struct pcmciamtd_dev {
 
 
 static dev_info_t dev_info = "pcmciamtd";
-static dev_link_t *dev_list;
 
 /* Module parameters */
 
@@ -782,8 +781,7 @@ static dev_link_t *pcmciamtd_attach(void
 	link->conf.Attributes = 0;
 	link->conf.IntType = INT_MEMORY;
 
-	link->next = dev_list;
-	dev_list = link;
+	link->next = NULL;
 
 	/* Register with Card Services */
 	client_reg.dev_info = &dev_info;
@@ -865,7 +863,6 @@ static void __exit exit_pcmciamtd(void)
 {
 	DEBUG(1, DRIVER_DESC " unloading");
 	pcmcia_unregister_driver(&pcmciamtd_driver);
-	BUG_ON(dev_list != NULL);
 }
 
 module_init(init_pcmciamtd);
diff -puN drivers/net/pcmcia/3c574_cs.c~pcmcia-remove-unused-dev_list-in-drivers drivers/net/pcmcia/3c574_cs.c
--- devel/drivers/net/pcmcia/3c574_cs.c~pcmcia-remove-unused-dev_list-in-drivers	2005-09-12 03:34:43.000000000 -0700
+++ devel-akpm/drivers/net/pcmcia/3c574_cs.c	2005-09-12 03:34:43.000000000 -0700
@@ -255,8 +255,6 @@ static dev_info_t dev_info = "3c574_cs";
 static dev_link_t *tc574_attach(void);
 static void tc574_detach(struct pcmcia_device *p_dev);
 
-static dev_link_t *dev_list;
-
 /*
 	tc574_attach() creates an "instance" of the driver, allocating
 	local data structures for one device.  The device is registered
@@ -308,8 +306,7 @@ static dev_link_t *tc574_attach(void)
 #endif
 
 	/* Register with Card Services */
-	link->next = dev_list;
-	dev_list = link;
+	link->next = NULL;
 	client_reg.dev_info = &dev_info;
 	client_reg.Version = 0x0210;
 	client_reg.event_callback_args.client_data = link;
@@ -336,24 +333,15 @@ static void tc574_detach(struct pcmcia_d
 {
 	dev_link_t *link = dev_to_instance(p_dev);
 	struct net_device *dev = link->priv;
-	dev_link_t **linkp;
 
 	DEBUG(0, "3c574_detach(0x%p)\n", link);
 
-	/* Locate device structure */
-	for (linkp = &dev_list; *linkp; linkp = &(*linkp)->next)
-		if (*linkp == link) break;
-	if (*linkp == NULL)
-		return;
-
 	if (link->dev)
 		unregister_netdev(dev);
 
 	if (link->state & DEV_CONFIG)
 		tc574_release(link);
 
-	/* Unlink device structure, free bits */
-	*linkp = link->next;
 	free_netdev(dev);
 } /* tc574_detach */
 
@@ -1310,7 +1298,6 @@ static int __init init_tc574(void)
 static void __exit exit_tc574(void)
 {
 	pcmcia_unregister_driver(&tc574_driver);
-	BUG_ON(dev_list != NULL);
 }
 
 module_init(init_tc574);
diff -puN drivers/net/pcmcia/3c589_cs.c~pcmcia-remove-unused-dev_list-in-drivers drivers/net/pcmcia/3c589_cs.c
--- devel/drivers/net/pcmcia/3c589_cs.c~pcmcia-remove-unused-dev_list-in-drivers	2005-09-12 03:34:43.000000000 -0700
+++ devel-akpm/drivers/net/pcmcia/3c589_cs.c	2005-09-12 03:34:43.000000000 -0700
@@ -166,8 +166,6 @@ static dev_info_t dev_info = "3c589_cs";
 static dev_link_t *tc589_attach(void);
 static void tc589_detach(struct pcmcia_device *p_dev);
 
-static dev_link_t *dev_list;
-
 /*======================================================================
 
     tc589_attach() creates an "instance" of the driver, allocating
@@ -222,8 +220,7 @@ static dev_link_t *tc589_attach(void)
     SET_ETHTOOL_OPS(dev, &netdev_ethtool_ops);
 
     /* Register with Card Services */
-    link->next = dev_list;
-    dev_list = link;
+    link->next = NULL;
     client_reg.dev_info = &dev_info;
     client_reg.Version = 0x0210;
     client_reg.event_callback_args.client_data = link;
@@ -250,15 +247,8 @@ static void tc589_detach(struct pcmcia_d
 {
     dev_link_t *link = dev_to_instance(p_dev);
     struct net_device *dev = link->priv;
-    dev_link_t **linkp;
-    
+
     DEBUG(0, "3c589_detach(0x%p)\n", link);
-    
-    /* Locate device structure */
-    for (linkp = &dev_list; *linkp; linkp = &(*linkp)->next)
-	if (*linkp == link) break;
-    if (*linkp == NULL)
-	return;
 
     if (link->dev)
 	unregister_netdev(dev);
@@ -266,8 +256,6 @@ static void tc589_detach(struct pcmcia_d
     if (link->state & DEV_CONFIG)
 	tc589_release(link);
 
-    /* Unlink device structure, free bits */
-    *linkp = link->next;
     free_netdev(dev);
 } /* tc589_detach */
 
@@ -1085,7 +1073,6 @@ static int __init init_tc589(void)
 static void __exit exit_tc589(void)
 {
 	pcmcia_unregister_driver(&tc589_driver);
-	BUG_ON(dev_list != NULL);
 }
 
 module_init(init_tc589);
diff -puN drivers/net/pcmcia/axnet_cs.c~pcmcia-remove-unused-dev_list-in-drivers drivers/net/pcmcia/axnet_cs.c
--- devel/drivers/net/pcmcia/axnet_cs.c~pcmcia-remove-unused-dev_list-in-drivers	2005-09-12 03:34:43.000000000 -0700
+++ devel-akpm/drivers/net/pcmcia/axnet_cs.c	2005-09-12 03:34:43.000000000 -0700
@@ -111,7 +111,6 @@ static dev_link_t *axnet_attach(void);
 static void axnet_detach(struct pcmcia_device *p_dev);
 
 static dev_info_t dev_info = "axnet_cs";
-static dev_link_t *dev_list;
 
 static void axdev_setup(struct net_device *dev);
 static void AX88190_init(struct net_device *dev, int startp);
@@ -177,8 +176,7 @@ static dev_link_t *axnet_attach(void)
     SET_ETHTOOL_OPS(dev, &netdev_ethtool_ops);
 
     /* Register with Card Services */
-    link->next = dev_list;
-    dev_list = link;
+    link->next = NULL;
     client_reg.dev_info = &dev_info;
     client_reg.Version = 0x0210;
     client_reg.event_callback_args.client_data = link;
@@ -205,24 +203,15 @@ static void axnet_detach(struct pcmcia_d
 {
     dev_link_t *link = dev_to_instance(p_dev);
     struct net_device *dev = link->priv;
-    dev_link_t **linkp;
 
     DEBUG(0, "axnet_detach(0x%p)\n", link);
 
-    /* Locate device structure */
-    for (linkp = &dev_list; *linkp; linkp = &(*linkp)->next)
-	if (*linkp == link) break;
-    if (*linkp == NULL)
-	return;
-
     if (link->dev)
 	unregister_netdev(dev);
 
     if (link->state & DEV_CONFIG)
 	axnet_release(link);
 
-    /* Unlink device structure, free bits */
-    *linkp = link->next;
     free_netdev(dev);
 } /* axnet_detach */
 
@@ -896,7 +885,6 @@ static int __init init_axnet_cs(void)
 static void __exit exit_axnet_cs(void)
 {
 	pcmcia_unregister_driver(&axnet_cs_driver);
-	BUG_ON(dev_list != NULL);
 }
 
 module_init(init_axnet_cs);
diff -puN drivers/net/pcmcia/com20020_cs.c~pcmcia-remove-unused-dev_list-in-drivers drivers/net/pcmcia/com20020_cs.c
--- devel/drivers/net/pcmcia/com20020_cs.c~pcmcia-remove-unused-dev_list-in-drivers	2005-09-12 03:34:43.000000000 -0700
+++ devel-akpm/drivers/net/pcmcia/com20020_cs.c	2005-09-12 03:34:43.000000000 -0700
@@ -128,8 +128,6 @@ static dev_info_t dev_info = "com20020_c
 static dev_link_t *com20020_attach(void);
 static void com20020_detach(struct pcmcia_device *p_dev);
 
-static dev_link_t *dev_list;
-
 /*====================================================================*/
 
 typedef struct com20020_dev_t {
@@ -196,8 +194,7 @@ static dev_link_t *com20020_attach(void)
     link->priv = info;
 
     /* Register with Card Services */
-    link->next = dev_list;
-    dev_list = link;
+    link->next = NULL;
     client_reg.dev_info = &dev_info;
     client_reg.Version = 0x0210;
     client_reg.event_callback_args.client_data = link;
@@ -230,26 +227,17 @@ static void com20020_detach(struct pcmci
 {
     dev_link_t *link = dev_to_instance(p_dev);
     struct com20020_dev_t *info = link->priv;
-    dev_link_t **linkp;
-    struct net_device *dev; 
-    
+    struct net_device *dev = info->dev;
+
     DEBUG(1,"detach...\n");
 
     DEBUG(0, "com20020_detach(0x%p)\n", link);
 
-    /* Locate device structure */
-    for (linkp = &dev_list; *linkp; linkp = &(*linkp)->next)
-        if (*linkp == link) break;
-    if (*linkp == NULL)
-        return;
-
-    dev = info->dev;
-
     if (link->dev) {
 	DEBUG(1,"unregister...\n");
 
 	unregister_netdev(dev);
-	    
+
 	/*
 	 * this is necessary because we register our IRQ separately
 	 * from card services.
@@ -263,7 +251,6 @@ static void com20020_detach(struct pcmci
 
     /* Unlink device structure, free bits */
     DEBUG(1,"unlinking...\n");
-    *linkp = link->next;
     if (link->priv)
     {
 	dev = info->dev;
@@ -507,7 +494,6 @@ static int __init init_com20020_cs(void)
 static void __exit exit_com20020_cs(void)
 {
 	pcmcia_unregister_driver(&com20020_cs_driver);
-	BUG_ON(dev_list != NULL);
 }
 
 module_init(init_com20020_cs);
diff -puN drivers/net/pcmcia/fmvj18x_cs.c~pcmcia-remove-unused-dev_list-in-drivers drivers/net/pcmcia/fmvj18x_cs.c
--- devel/drivers/net/pcmcia/fmvj18x_cs.c~pcmcia-remove-unused-dev_list-in-drivers	2005-09-12 03:34:43.000000000 -0700
+++ devel-akpm/drivers/net/pcmcia/fmvj18x_cs.c	2005-09-12 03:34:43.000000000 -0700
@@ -109,7 +109,6 @@ static void fjn_tx_timeout(struct net_de
 static struct ethtool_ops netdev_ethtool_ops;
 
 static dev_info_t dev_info = "fmvj18x_cs";
-static dev_link_t *dev_list;
 
 /*
     card type
@@ -284,8 +283,7 @@ static dev_link_t *fmvj18x_attach(void)
     SET_ETHTOOL_OPS(dev, &netdev_ethtool_ops);
     
     /* Register with Card Services */
-    link->next = dev_list;
-    dev_list = link;
+    link->next = NULL;
     client_reg.dev_info = &dev_info;
     client_reg.Version = 0x0210;
     client_reg.event_callback_args.client_data = link;
@@ -305,15 +303,8 @@ static void fmvj18x_detach(struct pcmcia
 {
     dev_link_t *link = dev_to_instance(p_dev);
     struct net_device *dev = link->priv;
-    dev_link_t **linkp;
-    
+
     DEBUG(0, "fmvj18x_detach(0x%p)\n", link);
-    
-    /* Locate device structure */
-    for (linkp = &dev_list; *linkp; linkp = &(*linkp)->next)
-	if (*linkp == link) break;
-    if (*linkp == NULL)
-	return;
 
     if (link->dev)
 	unregister_netdev(dev);
@@ -321,8 +312,6 @@ static void fmvj18x_detach(struct pcmcia
     if (link->state & DEV_CONFIG)
 	fmvj18x_release(link);
 
-    /* Unlink device structure, free pieces */
-    *linkp = link->next;
     free_netdev(dev);
 } /* fmvj18x_detach */
 
@@ -808,7 +797,6 @@ static int __init init_fmvj18x_cs(void)
 static void __exit exit_fmvj18x_cs(void)
 {
 	pcmcia_unregister_driver(&fmvj18x_cs_driver);
-	BUG_ON(dev_list != NULL);
 }
 
 module_init(init_fmvj18x_cs);
diff -puN drivers/net/pcmcia/ibmtr_cs.c~pcmcia-remove-unused-dev_list-in-drivers drivers/net/pcmcia/ibmtr_cs.c
--- devel/drivers/net/pcmcia/ibmtr_cs.c~pcmcia-remove-unused-dev_list-in-drivers	2005-09-12 03:34:43.000000000 -0700
+++ devel-akpm/drivers/net/pcmcia/ibmtr_cs.c	2005-09-12 03:34:43.000000000 -0700
@@ -116,8 +116,6 @@ static dev_info_t dev_info = "ibmtr_cs";
 static dev_link_t *ibmtr_attach(void);
 static void ibmtr_detach(struct pcmcia_device *p_dev);
 
-static dev_link_t *dev_list;
-
 /*====================================================================*/
 
 typedef struct ibmtr_dev_t {
@@ -186,8 +184,7 @@ static dev_link_t *ibmtr_attach(void)
     SET_ETHTOOL_OPS(dev, &netdev_ethtool_ops);
 
     /* Register with Card Services */
-    link->next = dev_list;
-    dev_list = link;
+    link->next = NULL;
     client_reg.dev_info = &dev_info;
     client_reg.Version = 0x0210;
     client_reg.event_callback_args.client_data = link;
@@ -219,19 +216,10 @@ static void ibmtr_detach(struct pcmcia_d
 {
     dev_link_t *link = dev_to_instance(p_dev);
     struct ibmtr_dev_t *info = link->priv;
-    dev_link_t **linkp;
-    struct net_device *dev;
+    struct net_device *dev = info->dev;
 
     DEBUG(0, "ibmtr_detach(0x%p)\n", link);
 
-    /* Locate device structure */
-    for (linkp = &dev_list; *linkp; linkp = &(*linkp)->next)
-        if (*linkp == link) break;
-    if (*linkp == NULL)
-        return;
-
-    dev = info->dev;
-
     if (link->dev)
 	unregister_netdev(dev);
 
@@ -242,10 +230,8 @@ static void ibmtr_detach(struct pcmcia_d
     if (link->state & DEV_CONFIG)
         ibmtr_release(link);
 
-    /* Unlink device structure, free bits */
-    *linkp = link->next;
     free_netdev(dev);
-    kfree(info); 
+    kfree(info);
 } /* ibmtr_detach */
 
 /*======================================================================
@@ -530,7 +516,6 @@ static int __init init_ibmtr_cs(void)
 static void __exit exit_ibmtr_cs(void)
 {
 	pcmcia_unregister_driver(&ibmtr_cs_driver);
-	BUG_ON(dev_list != NULL);
 }
 
 module_init(init_ibmtr_cs);
diff -puN drivers/net/pcmcia/nmclan_cs.c~pcmcia-remove-unused-dev_list-in-drivers drivers/net/pcmcia/nmclan_cs.c
--- devel/drivers/net/pcmcia/nmclan_cs.c~pcmcia-remove-unused-dev_list-in-drivers	2005-09-12 03:34:43.000000000 -0700
+++ devel-akpm/drivers/net/pcmcia/nmclan_cs.c	2005-09-12 03:34:43.000000000 -0700
@@ -389,7 +389,6 @@ DRV_NAME " " DRV_VERSION " (Roger C. Pao
 #endif
 
 static dev_info_t dev_info="nmclan_cs";
-static dev_link_t *dev_list;
 
 static char *if_names[]={
     "Auto", "10baseT", "BNC",
@@ -498,8 +497,7 @@ static dev_link_t *nmclan_attach(void)
 #endif
 
     /* Register with Card Services */
-    link->next = dev_list;
-    dev_list = link;
+    link->next = NULL;
     client_reg.dev_info = &dev_info;
     client_reg.Version = 0x0210;
     client_reg.event_callback_args.client_data = link;
@@ -525,24 +523,15 @@ static void nmclan_detach(struct pcmcia_
 {
     dev_link_t *link = dev_to_instance(p_dev);
     struct net_device *dev = link->priv;
-    dev_link_t **linkp;
 
     DEBUG(0, "nmclan_detach(0x%p)\n", link);
 
-    /* Locate device structure */
-    for (linkp = &dev_list; *linkp; linkp = &(*linkp)->next)
-	if (*linkp == link) break;
-    if (*linkp == NULL)
-	return;
-
     if (link->dev)
 	unregister_netdev(dev);
 
     if (link->state & DEV_CONFIG)
 	nmclan_release(link);
 
-    /* Unlink device structure, free bits */
-    *linkp = link->next;
     free_netdev(dev);
 } /* nmclan_detach */
 
@@ -1700,7 +1689,6 @@ static int __init init_nmclan_cs(void)
 static void __exit exit_nmclan_cs(void)
 {
 	pcmcia_unregister_driver(&nmclan_cs_driver);
-	BUG_ON(dev_list != NULL);
 }
 
 module_init(init_nmclan_cs);
diff -puN drivers/net/pcmcia/pcnet_cs.c~pcmcia-remove-unused-dev_list-in-drivers drivers/net/pcmcia/pcnet_cs.c
--- devel/drivers/net/pcmcia/pcnet_cs.c~pcmcia-remove-unused-dev_list-in-drivers	2005-09-12 03:34:43.000000000 -0700
+++ devel-akpm/drivers/net/pcmcia/pcnet_cs.c	2005-09-12 03:34:43.000000000 -0700
@@ -124,7 +124,6 @@ static dev_link_t *pcnet_attach(void);
 static void pcnet_detach(struct pcmcia_device *p_dev);
 
 static dev_info_t dev_info = "pcnet_cs";
-static dev_link_t *dev_list;
 
 /*====================================================================*/
 
@@ -272,8 +271,7 @@ static dev_link_t *pcnet_attach(void)
     dev->set_config = &set_config;
 
     /* Register with Card Services */
-    link->next = dev_list;
-    dev_list = link;
+    link->next = NULL;
     client_reg.dev_info = &dev_info;
     client_reg.Version = 0x0210;
     client_reg.event_callback_args.client_data = link;
@@ -300,24 +298,15 @@ static void pcnet_detach(struct pcmcia_d
 {
 	dev_link_t *link = dev_to_instance(p_dev);
 	struct net_device *dev = link->priv;
-	dev_link_t **linkp;
 
 	DEBUG(0, "pcnet_detach(0x%p)\n", link);
 
-	/* Locate device structure */
-	for (linkp = &dev_list; *linkp; linkp = &(*linkp)->next)
-		if (*linkp == link) break;
-	if (*linkp == NULL)
-		return;
-
 	if (link->dev)
 		unregister_netdev(dev);
 
 	if (link->state & DEV_CONFIG)
 		pcnet_release(link);
 
-	/* Unlink device structure, free bits */
-	*linkp = link->next;
 	free_netdev(dev);
 } /* pcnet_detach */
 
@@ -1858,7 +1847,6 @@ static void __exit exit_pcnet_cs(void)
 {
     DEBUG(0, "pcnet_cs: unloading\n");
     pcmcia_unregister_driver(&pcnet_driver);
-    BUG_ON(dev_list != NULL);
 }
 
 module_init(init_pcnet_cs);
diff -puN drivers/net/pcmcia/smc91c92_cs.c~pcmcia-remove-unused-dev_list-in-drivers drivers/net/pcmcia/smc91c92_cs.c
--- devel/drivers/net/pcmcia/smc91c92_cs.c~pcmcia-remove-unused-dev_list-in-drivers	2005-09-12 03:34:43.000000000 -0700
+++ devel-akpm/drivers/net/pcmcia/smc91c92_cs.c	2005-09-12 03:34:43.000000000 -0700
@@ -104,8 +104,6 @@ static const char *version =
 
 static dev_info_t dev_info = "smc91c92_cs";
 
-static dev_link_t *dev_list;
-
 struct smc_private {
     dev_link_t			link;
     spinlock_t			lock;
@@ -367,8 +365,7 @@ static dev_link_t *smc91c92_attach(void)
     smc->mii_if.reg_num_mask = 0x1f;
 
     /* Register with Card Services */
-    link->next = dev_list;
-    dev_list = link;
+    link->next = NULL;
     client_reg.dev_info = &dev_info;
     client_reg.Version = 0x0210;
     client_reg.event_callback_args.client_data = link;
@@ -395,24 +392,15 @@ static void smc91c92_detach(struct pcmci
 {
     dev_link_t *link = dev_to_instance(p_dev);
     struct net_device *dev = link->priv;
-    dev_link_t **linkp;
 
     DEBUG(0, "smc91c92_detach(0x%p)\n", link);
 
-    /* Locate device structure */
-    for (linkp = &dev_list; *linkp; linkp = &(*linkp)->next)
-	if (*linkp == link) break;
-    if (*linkp == NULL)
-	return;
-
     if (link->dev)
 	unregister_netdev(dev);
 
     if (link->state & DEV_CONFIG)
 	smc91c92_release(link);
 
-    /* Unlink device structure, free bits */
-    *linkp = link->next;
     free_netdev(dev);
 } /* smc91c92_detach */
 
@@ -2377,7 +2365,6 @@ static int __init init_smc91c92_cs(void)
 static void __exit exit_smc91c92_cs(void)
 {
 	pcmcia_unregister_driver(&smc91c92_cs_driver);
-	BUG_ON(dev_list != NULL);
 }
 
 module_init(init_smc91c92_cs);
diff -puN drivers/net/pcmcia/xirc2ps_cs.c~pcmcia-remove-unused-dev_list-in-drivers drivers/net/pcmcia/xirc2ps_cs.c
--- devel/drivers/net/pcmcia/xirc2ps_cs.c~pcmcia-remove-unused-dev_list-in-drivers	2005-09-12 03:34:43.000000000 -0700
+++ devel-akpm/drivers/net/pcmcia/xirc2ps_cs.c	2005-09-12 03:34:43.000000000 -0700
@@ -331,15 +331,7 @@ static dev_info_t dev_info = "xirc2ps_cs
  * device numbers are used to derive the corresponding array index.
  */
 
-static dev_link_t *dev_list;
-
 /****************
- * A dev_link_t structure has fields for most things that are needed
- * to keep track of a socket, but there will usually be some device
- * specific information that also needs to be kept track of.  The
- * 'priv' pointer in a dev_link_t structure can be used to point to
- * a device-specific private data structure, like this.
- *
  * A driver needs to provide a dev_node_t structure for each device
  * on a card.  In some cases, there is only one device per card (for
  * example, ethernet cards, modems).  In other cases, there may be
@@ -615,8 +607,7 @@ xirc2ps_attach(void)
 #endif
 
     /* Register with Card Services */
-    link->next = dev_list;
-    dev_list = link;
+    link->next = NULL;
     client_reg.dev_info = &dev_info;
     client_reg.Version = 0x0210;
     client_reg.event_callback_args.client_data = link;
@@ -641,27 +632,15 @@ xirc2ps_detach(struct pcmcia_device *p_d
 {
     dev_link_t *link = dev_to_instance(p_dev);
     struct net_device *dev = link->priv;
-    dev_link_t **linkp;
 
     DEBUG(0, "detach(0x%p)\n", link);
 
-    /* Locate device structure */
-    for (linkp = &dev_list; *linkp; linkp = &(*linkp)->next)
-	if (*linkp == link)
-	    break;
-    if (!*linkp) {
-	DEBUG(0, "detach(0x%p): dev_link lost\n", link);
-	return;
-    }
-
     if (link->dev)
 	unregister_netdev(dev);
 
     if (link->state & DEV_CONFIG)
 	xirc2ps_release(link);
 
-    /* Unlink device structure, free it */
-    *linkp = link->next;
     free_netdev(dev);
 } /* xirc2ps_detach */
 
@@ -2020,7 +1999,6 @@ static void __exit
 exit_xirc2ps_cs(void)
 {
 	pcmcia_unregister_driver(&xirc2ps_cs_driver);
-	BUG_ON(dev_list != NULL);
 }
 
 module_init(init_xirc2ps_cs);
diff -puN drivers/net/wireless/airo_cs.c~pcmcia-remove-unused-dev_list-in-drivers drivers/net/wireless/airo_cs.c
--- devel/drivers/net/wireless/airo_cs.c~pcmcia-remove-unused-dev_list-in-drivers	2005-09-12 03:34:43.000000000 -0700
+++ devel-akpm/drivers/net/wireless/airo_cs.c	2005-09-12 03:34:43.000000000 -0700
@@ -121,15 +121,7 @@ static dev_info_t dev_info = "airo_cs";
    device numbers are used to derive the corresponding array index.
 */
 
-static dev_link_t *dev_list = NULL;
-
 /*
-   A dev_link_t structure has fields for most things that are needed
-   to keep track of a socket, but there will usually be some device
-   specific information that also needs to be kept track of.  The
-   'priv' pointer in a dev_link_t structure can be used to point to
-   a device-specific private data structure, like this.
-
    A driver needs to provide a dev_node_t structure for each device
    on a card.  In some cases, there is only one device per card (for
    example, ethernet cards, modems).  In other cases, there may be
@@ -206,8 +198,7 @@ static dev_link_t *airo_attach(void)
 	link->priv = local;
 	
 	/* Register with Card Services */
-	link->next = dev_list;
-	dev_list = link;
+	link->next = NULL;
 	client_reg.dev_info = &dev_info;
 	client_reg.Version = 0x0210;
 	client_reg.event_callback_args.client_data = link;
@@ -233,31 +224,25 @@ static dev_link_t *airo_attach(void)
 static void airo_detach(struct pcmcia_device *p_dev)
 {
 	dev_link_t *link = dev_to_instance(p_dev);
-	dev_link_t **linkp;
-	
+
 	DEBUG(0, "airo_detach(0x%p)\n", link);
-	
+
 	/* Locate device structure */
-	for (linkp = &dev_list; *linkp; linkp = &(*linkp)->next)
-		if (*linkp == link) break;
-	if (*linkp == NULL)
-		return;
-	
+
 	if (link->state & DEV_CONFIG)
 		airo_release(link);
-	
+
 	if ( ((local_info_t*)link->priv)->eth_dev ) {
 		stop_airo_card( ((local_info_t*)link->priv)->eth_dev, 0 );
 	}
 	((local_info_t*)link->priv)->eth_dev = NULL;
 
 	/* Unlink device structure, free pieces */
-	*linkp = link->next;
 	if (link->priv) {
 		kfree(link->priv);
 	}
 	kfree(link);
-	
+
 } /* airo_detach */
 
 /*======================================================================
@@ -580,7 +565,6 @@ static int airo_cs_init(void)
 static void airo_cs_cleanup(void)
 {
 	pcmcia_unregister_driver(&airo_driver);
-	BUG_ON(dev_list != NULL);
 }
 
 /*
diff -puN drivers/net/wireless/atmel_cs.c~pcmcia-remove-unused-dev_list-in-drivers drivers/net/wireless/atmel_cs.c
--- devel/drivers/net/wireless/atmel_cs.c~pcmcia-remove-unused-dev_list-in-drivers	2005-09-12 03:34:43.000000000 -0700
+++ devel-akpm/drivers/net/wireless/atmel_cs.c	2005-09-12 03:34:43.000000000 -0700
@@ -129,15 +129,7 @@ static dev_info_t dev_info = "atmel_cs";
    device numbers are used to derive the corresponding array index.
 */
 
-static dev_link_t *dev_list = NULL;
-
 /*
-   A dev_link_t structure has fields for most things that are needed
-   to keep track of a socket, but there will usually be some device
-   specific information that also needs to be kept track of.  The
-   'priv' pointer in a dev_link_t structure can be used to point to
-   a device-specific private data structure, like this.
-
    A driver needs to provide a dev_node_t structure for each device
    on a card.  In some cases, there is only one device per card (for
    example, ethernet cards, modems).  In other cases, there may be
@@ -214,8 +206,7 @@ static dev_link_t *atmel_attach(void)
 	link->priv = local;
 	
 	/* Register with Card Services */
-	link->next = dev_list;
-	dev_list = link;
+	link->next = NULL;
 	client_reg.dev_info = &dev_info;
 	client_reg.Version = 0x0210;
 	client_reg.event_callback_args.client_data = link;
@@ -241,21 +232,12 @@ static dev_link_t *atmel_attach(void)
 static void atmel_detach(struct pcmcia_device *p_dev)
 {
 	dev_link_t *link = dev_to_instance(p_dev);
-	dev_link_t **linkp;
-	
+
 	DEBUG(0, "atmel_detach(0x%p)\n", link);
-	
-	/* Locate device structure */
-	for (linkp = &dev_list; *linkp; linkp = &(*linkp)->next)
-		if (*linkp == link) break;
-	if (*linkp == NULL)
-		return;
 
 	if (link->state & DEV_CONFIG)
 		atmel_release(link);
-		
-	/* Unlink device structure, free pieces */
-	*linkp = link->next;
+
 	if (link->priv)
 		kfree(link->priv);
 	kfree(link);
@@ -681,7 +663,6 @@ static int atmel_cs_init(void)
 static void atmel_cs_cleanup(void)
 {
         pcmcia_unregister_driver(&atmel_driver);
-	BUG_ON(dev_list != NULL);
 }
 
 /*
diff -puN drivers/net/wireless/hostap/hostap_cs.c~pcmcia-remove-unused-dev_list-in-drivers drivers/net/wireless/hostap/hostap_cs.c
--- devel/drivers/net/wireless/hostap/hostap_cs.c~pcmcia-remove-unused-dev_list-in-drivers	2005-09-12 03:34:43.000000000 -0700
+++ devel-akpm/drivers/net/wireless/hostap/hostap_cs.c	2005-09-12 03:34:43.000000000 -0700
@@ -25,7 +25,6 @@
 
 static char *version = PRISM2_VERSION " (Jouni Malinen <jkmaline@cc.hut.fi>)";
 static dev_info_t dev_info = "hostap_cs";
-static dev_link_t *dev_list = NULL;
 
 MODULE_AUTHOR("Jouni Malinen");
 MODULE_DESCRIPTION("Support for Intersil Prism2-based 802.11 wireless LAN "
@@ -552,8 +551,7 @@ static dev_link_t *prism2_attach(void)
 	link->conf.IntType = INT_MEMORY_AND_IO;
 
 	/* register with CardServices */
-	link->next = dev_list;
-	dev_list = link;
+	link->next = NULL;
 	client_reg.dev_info = &dev_info;
 	client_reg.Version = 0x0210;
 	client_reg.event_callback_args.client_data = link;
@@ -570,24 +568,13 @@ static dev_link_t *prism2_attach(void)
 static void prism2_detach(struct pcmcia_device *p_dev)
 {
 	dev_link_t *link = dev_to_instance(p_dev);
-	dev_link_t **linkp;
 
 	PDEBUG(DEBUG_FLOW, "prism2_detach\n");
 
-	for (linkp = &dev_list; *linkp; linkp = &(*linkp)->next)
-		if (*linkp == link)
-			break;
-	if (*linkp == NULL) {
-		printk(KERN_WARNING "%s: Attempt to detach non-existing "
-		       "PCMCIA client\n", dev_info);
-		return;
-	}
-
 	if (link->state & DEV_CONFIG) {
 		prism2_release((u_long)link);
 	}
 
-	*linkp = link->next;
 	/* release net devices */
 	if (link->priv) {
 		struct net_device *dev;
diff -puN drivers/net/wireless/netwave_cs.c~pcmcia-remove-unused-dev_list-in-drivers drivers/net/wireless/netwave_cs.c
--- devel/drivers/net/wireless/netwave_cs.c~pcmcia-remove-unused-dev_list-in-drivers	2005-09-12 03:34:43.000000000 -0700
+++ devel-akpm/drivers/net/wireless/netwave_cs.c	2005-09-12 03:34:43.000000000 -0700
@@ -233,17 +233,6 @@ static int netwave_ioctl(struct net_devi
 static void set_multicast_list(struct net_device *dev);
 
 /*
-   A linked list of "instances" of the skeleton device.  Each actual
-   PCMCIA card corresponds to one device instance, and is described
-   by one dev_link_t structure (defined in ds.h).
-
-   You may not want to use a linked list for this -- for example, the
-   memory card driver uses an array of dev_link_t pointers, where minor
-   device numbers are used to derive the corresponding array index.
-*/
-static dev_link_t *dev_list;
-
-/*
    A dev_link_t structure has fields for most things that are needed
    to keep track of a socket, but there will usually be some device
    specific information that also needs to be kept track of.  The
@@ -487,8 +476,7 @@ static dev_link_t *netwave_attach(void)
     link->irq.Instance = dev;
     
     /* Register with Card Services */
-    link->next = dev_list;
-    dev_list = link;
+    link->next = NULL;
     client_reg.dev_info = &dev_info;
     client_reg.Version = 0x0210;
     client_reg.event_callback_args.client_data = link;
@@ -512,37 +500,18 @@ static dev_link_t *netwave_attach(void)
  */
 static void netwave_detach(struct pcmcia_device *p_dev)
 {
-    dev_link_t *link = dev_to_instance(p_dev);
-    struct net_device *dev = link->priv;
-    dev_link_t **linkp;
+	dev_link_t *link = dev_to_instance(p_dev);
+	struct net_device *dev = link->priv;
 
-    DEBUG(0, "netwave_detach(0x%p)\n", link);
-  
-    /*
-	  If the device is currently configured and active, we won't
-	  actually delete it yet.  Instead, it is marked so that when
-	  the release() function is called, that will trigger a proper
-	  detach().
-	*/
-    if (link->state & DEV_CONFIG)
-	netwave_release(link);
-
-    /* Locate device structure */
-    for (linkp = &dev_list; *linkp; linkp = &(*linkp)->next)
-	if (*linkp == link) break;
-    if (*linkp == NULL)
-      {
-	DEBUG(1, "netwave_cs: detach fail, '%s' not in list\n",
-	      link->dev->dev_name);
-	return;
-      }
-
-    /* Unlink device structure, free pieces */
-    *linkp = link->next;
-    if (link->dev) 
-	unregister_netdev(dev);
-    free_netdev(dev);
-    
+	DEBUG(0, "netwave_detach(0x%p)\n", link);
+
+	if (link->state & DEV_CONFIG)
+		netwave_release(link);
+
+	if (link->dev)
+		unregister_netdev(dev);
+
+	free_netdev(dev);
 } /* netwave_detach */
 
 /*
@@ -1688,7 +1657,6 @@ static int __init init_netwave_cs(void)
 static void __exit exit_netwave_cs(void)
 {
 	pcmcia_unregister_driver(&netwave_driver);
-	BUG_ON(dev_list != NULL);
 }
 
 module_init(init_netwave_cs);
diff -puN drivers/net/wireless/orinoco_cs.c~pcmcia-remove-unused-dev_list-in-drivers drivers/net/wireless/orinoco_cs.c
--- devel/drivers/net/wireless/orinoco_cs.c~pcmcia-remove-unused-dev_list-in-drivers	2005-09-12 03:34:43.000000000 -0700
+++ devel-akpm/drivers/net/wireless/orinoco_cs.c	2005-09-12 03:34:43.000000000 -0700
@@ -86,12 +86,6 @@ struct orinoco_pccard {
 	unsigned long hard_reset_in_progress; 
 };
 
-/*
- * A linked list of "instances" of the device.  Each actual PCMCIA
- * card corresponds to one device instance, and is described by one
- * dev_link_t structure (defined in ds.h).
- */
-static dev_link_t *dev_list; /* = NULL */
 
 /********************************************************************/
 /* Function prototypes						    */
@@ -180,9 +174,7 @@ orinoco_cs_attach(void)
 	link->conf.IntType = INT_MEMORY_AND_IO;
 
 	/* Register with Card Services */
-	/* FIXME: need a lock? */
-	link->next = dev_list;
-	dev_list = link;
+	link->next = NULL;
 
 	client_reg.dev_info = &dev_info;
 	client_reg.Version = 0x0210; /* FIXME: what does this mean? */
@@ -207,21 +199,11 @@ orinoco_cs_attach(void)
 static void orinoco_cs_detach(struct pcmcia_device *p_dev)
 {
 	dev_link_t *link = dev_to_instance(p_dev);
-	dev_link_t **linkp;
 	struct net_device *dev = link->priv;
 
-	/* Locate device structure */
-	for (linkp = &dev_list; *linkp; linkp = &(*linkp)->next)
-		if (*linkp == link)
-			break;
-
-	BUG_ON(*linkp == NULL);
-
 	if (link->state & DEV_CONFIG)
 		orinoco_cs_release(link);
 
-	/* Unlink device structure, and free it */
-	*linkp = link->next;
 	DEBUG(0, PFX "detach: link=%p link->dev=%p\n", link, link->dev);
 	if (link->dev) {
 		DEBUG(0, PFX "About to unregister net device %p\n",
@@ -667,7 +649,6 @@ static void __exit
 exit_orinoco_cs(void)
 {
 	pcmcia_unregister_driver(&orinoco_driver);
-	BUG_ON(dev_list != NULL);
 }
 
 module_init(init_orinoco_cs);
diff -puN drivers/net/wireless/wavelan_cs.c~pcmcia-remove-unused-dev_list-in-drivers drivers/net/wireless/wavelan_cs.c
--- devel/drivers/net/wireless/wavelan_cs.c~pcmcia-remove-unused-dev_list-in-drivers	2005-09-12 03:34:43.000000000 -0700
+++ devel-akpm/drivers/net/wireless/wavelan_cs.c	2005-09-12 03:34:43.000000000 -0700
@@ -4634,8 +4634,7 @@ wavelan_attach(void)
   link->conf.IntType = INT_MEMORY_AND_IO;
 
   /* Chain drivers */
-  link->next = dev_list;
-  dev_list = link;
+  link->next = NULL;
 
   /* Allocate the generic data structure */
   dev = alloc_etherdev(sizeof(net_local));
@@ -4740,27 +4739,6 @@ wavelan_detach(struct pcmcia_device *p_d
       wv_pcmcia_release(link);
     }
 
-  /* Remove the interface data from the linked list */
-  if(dev_list == link)
-    dev_list = link->next;
-  else
-    {
-      dev_link_t *	prev = dev_list;
-
-      while((prev != (dev_link_t *) NULL) && (prev->next != link))
-	prev = prev->next;
-
-      if(prev == (dev_link_t *) NULL)
-	{
-#ifdef DEBUG_CONFIG_ERRORS
-	  printk(KERN_WARNING "wavelan_detach : Attempting to remove a nonexistent device.\n");
-#endif
-	  return;
-	}
-
-      prev->next = link->next;
-    }
-
   /* Free pieces */
   if(link->priv)
     {
diff -puN drivers/net/wireless/wavelan_cs.p.h~pcmcia-remove-unused-dev_list-in-drivers drivers/net/wireless/wavelan_cs.p.h
--- devel/drivers/net/wireless/wavelan_cs.p.h~pcmcia-remove-unused-dev_list-in-drivers	2005-09-12 03:34:43.000000000 -0700
+++ devel-akpm/drivers/net/wireless/wavelan_cs.p.h	2005-09-12 03:34:43.000000000 -0700
@@ -770,7 +770,6 @@ static int
 /**************************** VARIABLES ****************************/
 
 static dev_info_t dev_info = "wavelan_cs";
-static dev_link_t *dev_list = NULL;	/* Linked list of devices */
 
 /*
  * Parameters that can be set with 'insmod'
diff -puN drivers/parport/parport_cs.c~pcmcia-remove-unused-dev_list-in-drivers drivers/parport/parport_cs.c
--- devel/drivers/parport/parport_cs.c~pcmcia-remove-unused-dev_list-in-drivers	2005-09-12 03:34:43.000000000 -0700
+++ devel-akpm/drivers/parport/parport_cs.c	2005-09-12 03:34:43.000000000 -0700
@@ -95,7 +95,6 @@ static int parport_event(event_t event, 
 			 event_callback_args_t *args);
 
 static dev_info_t dev_info = "parport_cs";
-static dev_link_t *dev_list = NULL;
 
 /*======================================================================
 
@@ -129,8 +128,7 @@ static dev_link_t *parport_attach(void)
     link->conf.IntType = INT_MEMORY_AND_IO;
     
     /* Register with Card Services */
-    link->next = dev_list;
-    dev_list = link;
+    link->next = NULL;
     client_reg.dev_info = &dev_info;
     client_reg.Version = 0x0210;
     client_reg.event_callback_args.client_data = link;
@@ -156,21 +154,12 @@ static dev_link_t *parport_attach(void)
 static void parport_detach(struct pcmcia_device *p_dev)
 {
     dev_link_t *link = dev_to_instance(p_dev);
-    dev_link_t **linkp;
 
     DEBUG(0, "parport_detach(0x%p)\n", link);
 
-    /* Locate device structure */
-    for (linkp = &dev_list; *linkp; linkp = &(*linkp)->next)
-	if (*linkp == link) break;
-    if (*linkp == NULL)
-	return;
-
     if (link->state & DEV_CONFIG)
 	parport_cs_release(link);
 
-    /* Unlink, free device structure */
-    *linkp = link->next;
     kfree(link->priv);
 } /* parport_detach */
 
@@ -391,7 +380,6 @@ static int __init init_parport_cs(void)
 static void __exit exit_parport_cs(void)
 {
 	pcmcia_unregister_driver(&parport_cs_driver);
-	BUG_ON(dev_list != NULL);
 }
 
 module_init(init_parport_cs);
diff -puN drivers/scsi/pcmcia/fdomain_stub.c~pcmcia-remove-unused-dev_list-in-drivers drivers/scsi/pcmcia/fdomain_stub.c
--- devel/drivers/scsi/pcmcia/fdomain_stub.c~pcmcia-remove-unused-dev_list-in-drivers	2005-09-12 03:34:43.000000000 -0700
+++ devel-akpm/drivers/scsi/pcmcia/fdomain_stub.c	2005-09-12 03:34:43.000000000 -0700
@@ -87,8 +87,6 @@ static dev_link_t *fdomain_attach(void);
 static void fdomain_detach(struct pcmcia_device *p_dev);
 
 
-static dev_link_t *dev_list = NULL;
-
 static dev_info_t dev_info = "fdomain_cs";
 
 static dev_link_t *fdomain_attach(void)
@@ -116,8 +114,7 @@ static dev_link_t *fdomain_attach(void)
     link->conf.Present = PRESENT_OPTION;
 
     /* Register with Card Services */
-    link->next = dev_list;
-    dev_list = link;
+    link->next = NULL;
     client_reg.dev_info = &dev_info;
     client_reg.Version = 0x0210;
     client_reg.event_callback_args.client_data = link;
@@ -135,24 +132,14 @@ static dev_link_t *fdomain_attach(void)
 
 static void fdomain_detach(struct pcmcia_device *p_dev)
 {
-    dev_link_t *link = dev_to_instance(p_dev);
-    dev_link_t **linkp;
+	dev_link_t *link = dev_to_instance(p_dev);
 
-    DEBUG(0, "fdomain_detach(0x%p)\n", link);
-    
-    /* Locate device structure */
-    for (linkp = &dev_list; *linkp; linkp = &(*linkp)->next)
-	if (*linkp == link) break;
-    if (*linkp == NULL)
-	return;
-
-    if (link->state & DEV_CONFIG)
-	fdomain_release(link);
-
-    /* Unlink device structure, free bits */
-    *linkp = link->next;
-    kfree(link->priv);
-    
+	DEBUG(0, "fdomain_detach(0x%p)\n", link);
+
+	if (link->state & DEV_CONFIG)
+		fdomain_release(link);
+
+	kfree(link->priv);
 } /* fdomain_detach */
 
 /*====================================================================*/
@@ -324,7 +311,6 @@ static int __init init_fdomain_cs(void)
 static void __exit exit_fdomain_cs(void)
 {
 	pcmcia_unregister_driver(&fdomain_cs_driver);
-	BUG_ON(dev_list != NULL);
 }
 
 module_init(init_fdomain_cs);
diff -puN drivers/scsi/pcmcia/nsp_cs.c~pcmcia-remove-unused-dev_list-in-drivers drivers/scsi/pcmcia/nsp_cs.c
--- devel/drivers/scsi/pcmcia/nsp_cs.c~pcmcia-remove-unused-dev_list-in-drivers	2005-09-12 03:34:43.000000000 -0700
+++ devel-akpm/drivers/scsi/pcmcia/nsp_cs.c	2005-09-12 03:34:43.000000000 -0700
@@ -104,7 +104,6 @@ static Scsi_Host_Template nsp_driver_tem
 #endif
 };
 
-static dev_link_t *dev_list = NULL;
 static dev_info_t dev_info  = {"nsp_cs"};
 
 static nsp_hw_data nsp_data_base; /* attach <-> detect glue */
@@ -1638,8 +1637,7 @@ static dev_link_t *nsp_cs_attach(void)
 
 
 	/* Register with Card Services */
-	link->next               = dev_list;
-	dev_list                 = link;
+	link->next               = NULL;
 	client_reg.dev_info	 = &dev_info;
 	client_reg.Version	 = 0x0210;
 	client_reg.event_callback_args.client_data = link;
@@ -1665,30 +1663,16 @@ static dev_link_t *nsp_cs_attach(void)
 static void nsp_cs_detach(struct pcmcia_device *p_dev)
 {
 	dev_link_t *link = dev_to_instance(p_dev);
-	dev_link_t **linkp;
 
 	nsp_dbg(NSP_DEBUG_INIT, "in, link=0x%p", link);
 
-	/* Locate device structure */
-	for (linkp = &dev_list; *linkp; linkp = &(*linkp)->next) {
-		if (*linkp == link) {
-			break;
-		}
-	}
-	if (*linkp == NULL) {
-		return;
-	}
-
 	if (link->state & DEV_CONFIG) {
 		((scsi_info_t *)link->priv)->stop = 1;
 		nsp_cs_release(link);
 	}
 
-	/* Unlink device structure, free bits */
-	*linkp = link->next;
 	kfree(link->priv);
 	link->priv = NULL;
-
 } /* nsp_cs_detach */
 
 
@@ -2168,7 +2152,6 @@ static void __exit nsp_cs_exit(void)
 
 #if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,68))
 	pcmcia_unregister_driver(&nsp_driver);
-	BUG_ON(dev_list != NULL);
 #else
 	unregister_pcmcia_driver(&dev_info);
 	/* XXX: this really needs to move into generic code.. */
diff -puN drivers/scsi/pcmcia/qlogic_stub.c~pcmcia-remove-unused-dev_list-in-drivers drivers/scsi/pcmcia/qlogic_stub.c
--- devel/drivers/scsi/pcmcia/qlogic_stub.c~pcmcia-remove-unused-dev_list-in-drivers	2005-09-12 03:34:43.000000000 -0700
+++ devel-akpm/drivers/scsi/pcmcia/qlogic_stub.c	2005-09-12 03:34:43.000000000 -0700
@@ -104,8 +104,6 @@ static dev_link_t *qlogic_attach(void);
 static void qlogic_detach(struct pcmcia_device *p_dev);
 
 
-static dev_link_t *dev_list = NULL;
-
 static dev_info_t dev_info = "qlogic_cs";
 
 static struct Scsi_Host *qlogic_detect(Scsi_Host_Template *host,
@@ -190,8 +188,7 @@ static dev_link_t *qlogic_attach(void)
 	link->conf.Present = PRESENT_OPTION;
 
 	/* Register with Card Services */
-	link->next = dev_list;
-	dev_list = link;
+	link->next = NULL;
 	client_reg.dev_info = &dev_info;
 	client_reg.Version = 0x0210;
 	client_reg.event_callback_args.client_data = link;
@@ -210,22 +207,12 @@ static dev_link_t *qlogic_attach(void)
 static void qlogic_detach(struct pcmcia_device *p_dev)
 {
 	dev_link_t *link = dev_to_instance(p_dev);
-	dev_link_t **linkp;
 
 	DEBUG(0, "qlogic_detach(0x%p)\n", link);
 
-	/* Locate device structure */
-	for (linkp = &dev_list; *linkp; linkp = &(*linkp)->next)
-		if (*linkp == link)
-			break;
-	if (*linkp == NULL)
-		return;
-
 	if (link->state & DEV_CONFIG)
 		qlogic_release(link);
 
-	/* Unlink device structure, free bits */
-	*linkp = link->next;
 	kfree(link->priv);
 
 }				/* qlogic_detach */
@@ -439,7 +426,6 @@ static int __init init_qlogic_cs(void)
 static void __exit exit_qlogic_cs(void)
 {
 	pcmcia_unregister_driver(&qlogic_cs_driver);
-	BUG_ON(dev_list != NULL);
 }
 
 MODULE_AUTHOR("Tom Zerucha, Michael Griffith");
diff -puN drivers/scsi/pcmcia/sym53c500_cs.c~pcmcia-remove-unused-dev_list-in-drivers drivers/scsi/pcmcia/sym53c500_cs.c
--- devel/drivers/scsi/pcmcia/sym53c500_cs.c~pcmcia-remove-unused-dev_list-in-drivers	2005-09-12 03:34:43.000000000 -0700
+++ devel-akpm/drivers/scsi/pcmcia/sym53c500_cs.c	2005-09-12 03:34:43.000000000 -0700
@@ -232,7 +232,6 @@ enum Phase {
 *  Global (within this module) variables other than
 *  sym53c500_driver_template (the scsi_host_template).
 */
-static dev_link_t *dev_list;
 static dev_info_t dev_info = "sym53c500_cs";
 
 /* ================================================================== */
@@ -930,22 +929,12 @@ static void
 SYM53C500_detach(struct pcmcia_device *p_dev)
 {
 	dev_link_t *link = dev_to_instance(p_dev);
-	dev_link_t **linkp;
 
 	DEBUG(0, "SYM53C500_detach(0x%p)\n", link);
 
-	/* Locate device structure */
-	for (linkp = &dev_list; *linkp; linkp = &(*linkp)->next)
-		if (*linkp == link)
-			break;
-	if (*linkp == NULL)
-		return;
-
 	if (link->state & DEV_CONFIG)
 		SYM53C500_release(link);
 
-	/* Unlink device structure, free bits. */
-	*linkp = link->next;
 	kfree(link->priv);
 	link->priv = NULL;
 } /* SYM53C500_detach */
@@ -978,8 +967,7 @@ SYM53C500_attach(void)
 	link->conf.Present = PRESENT_OPTION;
 
 	/* Register with Card Services */
-	link->next = dev_list;
-	dev_list = link;
+	link->next = NULL;
 	client_reg.dev_info = &dev_info;
 	client_reg.Version = 0x0210;
 	client_reg.event_callback_args.client_data = link;
diff -puN drivers/serial/serial_cs.c~pcmcia-remove-unused-dev_list-in-drivers drivers/serial/serial_cs.c
--- devel/drivers/serial/serial_cs.c~pcmcia-remove-unused-dev_list-in-drivers	2005-09-12 03:34:43.000000000 -0700
+++ devel-akpm/drivers/serial/serial_cs.c	2005-09-12 03:34:43.000000000 -0700
@@ -122,8 +122,6 @@ static dev_info_t dev_info = "serial_cs"
 static dev_link_t *serial_attach(void);
 static void serial_detach(struct pcmcia_device *p_dev);
 
-static dev_link_t *dev_list = NULL;
-
 /*======================================================================
 
     After a card is removed, serial_remove() will unregister
@@ -234,8 +232,7 @@ static dev_link_t *serial_attach(void)
 	link->conf.IntType = INT_MEMORY_AND_IO;
 
 	/* Register with Card Services */
-	link->next = dev_list;
-	dev_list = link;
+	link->next = NULL; /* not needed */
 	client_reg.dev_info = &dev_info;
 	client_reg.Version = 0x0210;
 	client_reg.event_callback_args.client_data = link;
@@ -262,17 +259,9 @@ static void serial_detach(struct pcmcia_
 {
 	dev_link_t *link = dev_to_instance(p_dev);
 	struct serial_info *info = link->priv;
-	dev_link_t **linkp;
 
 	DEBUG(0, "serial_detach(0x%p)\n", link);
 
-	/* Locate device structure */
-	for (linkp = &dev_list; *linkp; linkp = &(*linkp)->next)
-		if (*linkp == link)
-			break;
-	if (*linkp == NULL)
-		return;
-
 	/*
 	 * Ensure any outstanding scheduled tasks are completed.
 	 */
@@ -283,8 +272,7 @@ static void serial_detach(struct pcmcia_
 	 */
 	serial_remove(link);
 
-	/* Unlink device structure, free bits */
-	*linkp = link->next;
+	/* free bits */
 	kfree(info);
 }
 
@@ -870,7 +858,6 @@ static int __init init_serial_cs(void)
 static void __exit exit_serial_cs(void)
 {
 	pcmcia_unregister_driver(&serial_cs_driver);
-	BUG_ON(dev_list != NULL);
 }
 
 module_init(init_serial_cs);
diff -puN drivers/telephony/ixj_pcmcia.c~pcmcia-remove-unused-dev_list-in-drivers drivers/telephony/ixj_pcmcia.c
--- devel/drivers/telephony/ixj_pcmcia.c~pcmcia-remove-unused-dev_list-in-drivers	2005-09-12 03:34:43.000000000 -0700
+++ devel-akpm/drivers/telephony/ixj_pcmcia.c	2005-09-12 03:34:43.000000000 -0700
@@ -40,7 +40,6 @@ static void ixj_config(dev_link_t * link
 static void ixj_cs_release(dev_link_t * link);
 static int ixj_event(event_t event, int priority, event_callback_args_t * args);
 static dev_info_t dev_info = "ixj_cs";
-static dev_link_t *dev_list = NULL;
 
 static dev_link_t *ixj_attach(void)
 {
@@ -65,8 +64,7 @@ static dev_link_t *ixj_attach(void)
 	}
 	memset(link->priv, 0, sizeof(struct ixj_info_t));
 	/* Register with Card Services */
-	link->next = dev_list;
-	dev_list = link;
+	link->next = NULL;
 	client_reg.dev_info = &dev_info;
 	client_reg.Version = 0x0210;
 	client_reg.event_callback_args.client_data = link;
@@ -82,20 +80,13 @@ static dev_link_t *ixj_attach(void)
 static void ixj_detach(struct pcmcia_device *p_dev)
 {
 	dev_link_t *link = dev_to_instance(p_dev);
-	dev_link_t **linkp;
 
 	DEBUG(0, "ixj_detach(0x%p)\n", link);
-	for (linkp = &dev_list; *linkp; linkp = &(*linkp)->next)
-		if (*linkp == link)
-			break;
-	if (*linkp == NULL)
-		return;
+
 	link->state &= ~DEV_RELEASE_PENDING;
 	if (link->state & DEV_CONFIG)
 		ixj_cs_release(link);
 
-	/* Unlink device structure, free bits */
-	*linkp = link->next;
         kfree(link->priv);
         kfree(link);
 }
@@ -314,7 +305,6 @@ static int __init ixj_pcmcia_init(void)
 static void ixj_pcmcia_exit(void)
 {
 	pcmcia_unregister_driver(&ixj_driver);
-	BUG_ON(dev_list != NULL);
 }
 
 module_init(ixj_pcmcia_init);
diff -puN drivers/usb/host/sl811_cs.c~pcmcia-remove-unused-dev_list-in-drivers drivers/usb/host/sl811_cs.c
--- devel/drivers/usb/host/sl811_cs.c~pcmcia-remove-unused-dev_list-in-drivers	2005-09-12 03:34:43.000000000 -0700
+++ devel-akpm/drivers/usb/host/sl811_cs.c	2005-09-12 03:34:43.000000000 -0700
@@ -65,8 +65,6 @@ module_param(pc_debug, int, 0644);
 
 static const char driver_name[DEV_NAME_LEN]  = "sl811_cs";
 
-static dev_link_t *dev_list = NULL;
-
 typedef struct local_info_t {
 	dev_link_t		link;
 	dev_node_t		node;
@@ -141,24 +139,13 @@ static int sl811_hc_init(struct device *
 static void sl811_cs_detach(struct pcmcia_device *p_dev)
 {
 	dev_link_t *link = dev_to_instance(p_dev);
-	dev_link_t **linkp;
 
 	DBG(0, "sl811_cs_detach(0x%p)\n", link);
 
-	/* Locate device structure */
-	for (linkp = &dev_list; *linkp; linkp = &(*linkp)->next) {
-		if (*linkp == link)
-			break;
-	}
-	if (*linkp == NULL)
-		return;
-
 	link->state &= ~DEV_PRESENT;
 	if (link->state & DEV_CONFIG)
 		sl811_cs_release(link);
 
-	/* Unlink device structure, and free it */
-	*linkp = link->next;
 	/* This points to the parent local_info_t struct */
 	kfree(link->priv);
 }
@@ -376,8 +363,7 @@ static dev_link_t *sl811_cs_attach(void)
 	link->conf.IntType = INT_MEMORY_AND_IO;
 
 	/* Register with Card Services */
-	link->next = dev_list;
-	dev_list = link;
+	link->next = NULL;
 	client_reg.dev_info = (dev_info_t *) &driver_name;
 	client_reg.Attributes = INFO_IO_CLIENT | INFO_CARD_SHARE;
 	client_reg.Version = 0x0210;
diff -puN sound/pcmcia/pdaudiocf/pdaudiocf.c~pcmcia-remove-unused-dev_list-in-drivers sound/pcmcia/pdaudiocf/pdaudiocf.c
--- devel/sound/pcmcia/pdaudiocf/pdaudiocf.c~pcmcia-remove-unused-dev_list-in-drivers	2005-09-12 03:34:43.000000000 -0700
+++ devel-akpm/sound/pcmcia/pdaudiocf/pdaudiocf.c	2005-09-12 03:34:43.000000000 -0700
@@ -54,7 +54,6 @@ MODULE_PARM_DESC(enable, "Enable " CARD_
 
 static dev_info_t dev_info = "snd-pdaudiocf";
 static snd_card_t *card_list[SNDRV_CARDS];
-static dev_link_t *dev_list;
 
 /*
  * prototypes
@@ -161,8 +160,7 @@ static dev_link_t *snd_pdacf_attach(void
 	link->conf.Present = PRESENT_OPTION;
 
 	/* Chain drivers */
-	link->next = dev_list;
-	dev_list = link;
+	link->next = NULL;
 
 	/* Register with Card Services */
 	client_reg.dev_info = &dev_info;
@@ -231,16 +229,7 @@ static void snd_pdacf_detach(struct pcmc
 	pdacf_t *chip = link->priv;
 
 	snd_printdd(KERN_DEBUG "pdacf_detach called\n");
-	/* Remove the interface data from the linked list */
-	{
-		dev_link_t **linkp;
-		/* Locate device structure */
-		for (linkp = &dev_list; *linkp; linkp = &(*linkp)->next)
-			if (*linkp == link)
-				break;
-		if (*linkp)
-			*linkp = link->next;
-	}
+
 	if (chip->chip_status & PDAUDIOCF_STAT_IS_CONFIGURED)
 		snd_pdacf_powerdown(chip);
 	chip->chip_status |= PDAUDIOCF_STAT_IS_STALE; /* to be sure */
@@ -403,7 +392,6 @@ static int __init init_pdacf(void)
 static void __exit exit_pdacf(void)
 {
 	pcmcia_unregister_driver(&pdacf_cs_driver);
-	BUG_ON(dev_list != NULL);
 }
 
 module_init(init_pdacf);
diff -puN sound/pcmcia/vx/vxpocket.c~pcmcia-remove-unused-dev_list-in-drivers sound/pcmcia/vx/vxpocket.c
--- devel/sound/pcmcia/vx/vxpocket.c~pcmcia-remove-unused-dev_list-in-drivers	2005-09-12 03:34:43.000000000 -0700
+++ devel-akpm/sound/pcmcia/vx/vxpocket.c	2005-09-12 03:34:43.000000000 -0700
@@ -55,7 +55,6 @@ MODULE_PARM_DESC(ibl, "Capture IBL size 
  */
 
 static unsigned int card_alloc;
-static dev_link_t *dev_list;		/* Linked list of devices */
 static dev_info_t dev_info = "snd-vxpocket";
 
 
@@ -412,8 +411,7 @@ static dev_link_t *vxpocket_attach(void)
 	card_alloc |= 1 << i;
 
 	/* Chain drivers */
-	vxp->link.next = dev_list;
-	dev_list = &vxp->link;
+	vxp->link.next = NULL;
 
 	return &vxp->link;
 }
@@ -423,7 +421,6 @@ static void vxpocket_detach(struct pcmci
 	dev_link_t *link = dev_to_instance(p_dev);
 	struct snd_vxpocket *vxp;
 	vx_core_t *chip;
-	dev_link_t **linkp;
 
 	if (! link)
 		return;
@@ -432,13 +429,6 @@ static void vxpocket_detach(struct pcmci
 	chip = (vx_core_t *)vxp;
 	card_alloc &= ~(1 << vxp->index);
 
-	/* Remove the interface data from the linked list */
-	for (linkp = &dev_list; *linkp; linkp = &(*linkp)->next)
-		if (*linkp == link) {
-			*linkp = link->next;
-			break;
-		}
-
 	chip->chip_status |= VX_STAT_IS_STALE; /* to be sure */
 	snd_card_disconnect(chip->card);
 	vxpocket_release(link);
@@ -478,7 +468,6 @@ static int __init init_vxpocket(void)
 static void __exit exit_vxpocket(void)
 {
 	pcmcia_unregister_driver(&vxp_cs_driver);
-	BUG_ON(dev_list != NULL);
 }
 
 module_init(init_vxpocket);
_
